package edu.ucla.iBeaconNav;

import java.util.List;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Collection;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import android.app.Notification;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.os.IBinder;
import android.os.Looper;
import android.os.Messenger;
import android.os.RemoteException;
import android.preference.PreferenceManager;

import com.radiusnetworks.ibeacon.*;
import com.radiusnetworks.ibeacon.client.*;
import com.radiusnetworks.ibeacon.service.*;

public class Beacon implements IBeaconConsumer, RangeNotifier, MonitorNotifier
{
	/* Private data */
	private Task           task;

	private IBeaconManager ibeacon;
	private String         uuid;
	private HashMap<Integer,LatLon> locations;

	/* Public methods */
	public Beacon(Task task) {
		IBeaconManager.LOG_DEBUG = true;

		// Setup object
		this.uuid = "iBeaconNav";
		this.task = task;

		// Initialize iBeacons
		this.locations = new HashMap<Integer,LatLon>();
		this.addIBeacon(1, 1, 34.0722, -118.4441);
		this.addIBeacon(1, 2, 34.0722, -118.4441);
		this.addIBeacon(1, 3, 34.0722, -118.4441);
		this.addIBeacon(1, 5, 34.0722, -118.4441);
		this.addIBeacon(1, 5, 34.0722, -118.4441);
	}

	public void connect() {
		Util.debug("Task: handle - connect");
		this.task.notify("Connected", android.R.drawable.presence_online);
		this.ibeacon = IBeaconManager.getInstanceForApplication(this.task);
		this.ibeacon.bind(this);
		//this.ibeacon.setBackgroundMode(this, false);
	}

	public void disconnect() {
		Util.debug("Task: handle - register");
		this.ibeacon.unBind(this);
		//this.ibeacon.setBackgroundMode(this, true);
	}

	/* Private methods */
	private void addIBeacon(int major, int minor, double lat, double lon) {
		IBeacon ib  = new IBeacon(this.uuid, major, minor);
		LatLon  loc = new LatLon(lat, lon);
		this.locations.put(ib.getMinor(), loc);
	}

	/* IBeaconConsumer Methods
	 *   Pass most of these off to Task,
	 *   I don't even know why they're here */
	@Override
	public boolean bindService(Intent intent, ServiceConnection connection, int mode) {
		return this.task.bindService(intent, connection, mode);
	}

	@Override
	public Context getApplicationContext() {
		return this.task.getApplicationContext();
	}

	@Override
	public void unbindService(ServiceConnection connection)  {
		this.task.unbindService(connection);
	}

	@Override
	public void onIBeaconServiceConnect() {
		Util.debug("Task: onIBeaconServiceConnect");

		this.ibeacon.setRangeNotifier(this);
		this.ibeacon.setMonitorNotifier(this);

		try {
			Region region = new Region("iBeaconNavMonitoringId", null, null, null);
			this.ibeacon.startRangingBeaconsInRegion(region);
			this.ibeacon.startMonitoringBeaconsInRegion(region);
		} catch (RemoteException e) {
			Util.debug("Task: onIBeaconServiceConnect - error");
		}
	}

	/* RangeNotifier Methods */
        @Override
        public void didRangeBeaconsInRegion(Collection<IBeacon> iBeacons, Region region) {
		Util.debug("Task: didRangeBeaconsInRegion - " + iBeacons.size() + " beacons");
		for (IBeacon ib : iBeacons) {
			if (this.locations.containsKey(ib.getMinor())) {
				LatLon loc = this.locations.get(ib.getMinor());
				float[] data = new float[4];
				data[0] = CMD.Data.BEACON.ordinal();
				data[1] = (float)loc.lat;
				data[2] = (float)loc.lon;
				data[3] = (float)ib.getAccuracy();
				this.task.tellMain(CMD.Response.SHOWDATA, data);
			} else {
				Util.debug("Unknown beacon:" +
						" "   + ib.getProximityUuid() +
						":"   + ib.getMajor() +
						":"   + ib.getMinor() +
						" @ " + ib.getAccuracy());
			}
		}
        }

	/* MonitorNotifier Methods */
	@Override
	public void didEnterRegion(Region region) {
		Util.debug("Task: didEnterRegion");
	}

	@Override
	public void didExitRegion(Region region) {
		Util.debug("Task: didExitRegion");
	}

	@Override
	public void didDetermineStateForRegion(int state, Region region) {
		Util.debug("Task: didDetermineStateForRegion");
	}

	/* Location class */
	class LatLon {
		public double lat = 0;
		public double lon = 0;

		public LatLon(double lat, double lon) {
			this.lat = lat;
			this.lon = lon;
		}
	}
}
